
#Mixed Trophic Impact
MTI<- function(flowM, inputM, expM, respM, liveM, storM, relative=FALSE){
  #build enaR model
  enaRmodel<-pack(flow=flowM,input=inputM,export=expM,respiration=respM,living=liveM,storage=storM)
  #calculate MTI matrix
  mti<-mixedTrophicImpacts(enaRmodel,eigen.check=FALSE)
  mtiMatrix<-t(mti$M)
  rownames(mtiMatrix)<-seq(from=1,to=nrow(mtiMatrix))
  colnam<-colnames(mtiMatrix)
  colnames(mtiMatrix)<-paste(seq(from=1,to=nrow(mtiMatrix)),colnam,sep=".")
  if(relative){
    mtiRelMatrix<-mtiMatrix
    for(i in 1:nrow(mtiRelMatrix)){
      for(j in 1:ncol(mtiRelMatrix)){
        mtiRelMatrix[i,j]<-mtiMatrix[i,j]/storM[j,]
      }
    }
    mtiMatrix<-mtiRelMatrix
  }
  eigenValError<-is.na(mixedTrophicImpacts(enaRmodel)$M)
  return(list(enaRmodel=enaRmodel, mtiM=mtiMatrix,eigenValErr=eigenValError)) 
}

keystoneness<-function(enaRmodel,storM,detritusName="DETRITUS",discardName="DISCARD",flowM){

  #calculation of keystoneness
  mti<-mixedTrophicImpacts(enaRmodel,eigen.check=FALSE)
  mtiM<-mti$M
  eps<-rep(0,nrow(mtiM))
  for(i in 1:nrow(mtiM)){
    eps[i]<-sqrt(sum(mtiM[i,]^2) - mtiM[i,i]^2)
  }
  #trick
  eps<-storM+eps-storM
  
  eps_w<-rep(0,nrow(mtiM))
  for(i in 1:nrow(mtiM)){
    eps_w[i]<-sqrt(sum((mtiM[i,]/storM)^2) - (mtiM[i,i]/storM[i,])^2)
  }
  eps_w<-storM+eps_w-storM
  totBiomass<-sum(storM)
  p<-storM/totBiomass
  #libralato et al. 2006
  ks<-log(eps*(1-p))
 
  
  #social network analysis
  degC<-degree(enaRmodel)
  
  btwn<-betweenness(enaRmodel,diag=TRUE,cmode="proximaltar")
  btwn<-betweenness(flowM,diag=TRUE,cmode="proximaltar")
    
  closeC<-closeness(enaRmodel,diag=TRUE)
  closeC<-closeness(flowM,diag=TRUE)
  
  flowB<-flowbet(enaRmodel,diag=TRUE,cmode="fracflow")
  flowB<-flowbet(flowM,diag=TRUE,cmode="fracflow")
  
  
  eigenC<-eigenCentrality(unpack(enaRmodel)$F)$AEVC
  
  eigenValError<-is.na(mixedTrophicImpacts(enaRmodel)$M)
  filter=rownames(ks)!=detritusName&rownames(ks)!=discardName
  epsNoFD<-eps[filter,]
  ksNoFD<-ks[filter,]
  degCNoFD<-degC[filter]
  btwnNoFD<-btwn[filter]
  closeCNoFD<-closeC[filter]
  flowBNoFD<-flowB[filter]
  eigenCNoFD<-eigenC[filter]
  
  return(list(eps=eps,ks=ks,degC=degC,btwn=btwn,closeC=closeC, flowB=flowB, eigenC=eigenC, 
              epsNoFD=epsNoFD,ksNoFD=ksNoFD,degCNoFD=degCNoFD,btwnNoFD=btwnNoFD,closeCNoFD=closeCNoFD,
              flowBNoFD=flowBNoFD,eigenCNoFD=eigenCNoFD,
              eigenValErr=eigenValError))
}

addFakeCompartment<-function(flow,compName){
  flowM<-matrix(0,nrow=nrow(flow)+1,ncol=ncol(flow)+1)
  for(i in 1:nrow(flow))
    for(j in 1:ncol(flow))
      flowM[i+1,j+1]<-flow[i,j]
  colnames(flowM)<-c(compName,colnames(flow))
  rownames(flowM)<-c(compName,rownames(flow))
  return(flowM)
}